export default class CookieManager {
	private cookies: Record<string, string> = {}
	public constructor(data?: string | Record<string, string>) {
		if (typeof data === 'string') {
			data.split(';').forEach((keyValuePair) => {
				const [key, value] = keyValuePair.split('=')
				if (!key || !value) {
					return
				}
				this.cookies[key.trim()] = value.trim().replace(/%3B/g, ';')
			})
		} else if (typeof data === 'object') {
			this.cookies = data
		}
	}

	public static addCookie(res: ResponseInit & { readonly headers: Headers; }, cookie: {
		key: string
		value: string
		expire?: string
		maxAge?: number
		domain?: string
		path?: string
		secure?: boolean
		httpOnly?: boolean
		sameSite?: 'Lax' | 'None' | 'Strict'}
	) {
		const items: Array<string> = [`${cookie.key}=${cookie.value.replace(/;/g, '%3B')}`]
		if (cookie.expire) {
			items.push(`Expires=${cookie.expire}`)
		}
		if (cookie.maxAge) {
			items.push(`Max-Age=${cookie.maxAge}`)
		}
		if (cookie.domain) {
			items.push(`Domain=${cookie.domain}`)
		}
		if (cookie.path) {
			items.push(`Path=${cookie.path}`)
		}
		if (cookie.secure) {
			items.push('Secure')
		}
		if (cookie.httpOnly) {
			items.push('HttpOnly')
		}
		if (cookie.sameSite) {
			items.push(`SameSite=${cookie.sameSite}`)
		}
		res.headers.append('Set-Cookie', items.join('; '))
	}

	public get(key: string): string | undefined {
		return this.cookies[key]
	}

}
