import { objectLoop } from '@dzeio/object-util'
import URLManager from '@dzeio/url-manager'
import { defineMiddleware } from "astro/middleware"
import { buildRFC7807 } from '../libs/RFCs/RFC7807'
import { Permission, validateAuth } from '../libs/validateAuth'

const endpointsPermissions: Record<string, Permission> = {
	'/api/v1/users/[userId]/configs/[configId]/files/[fileName]': {
		api: true,
		cookie: true,
		name: 'configs.get'
	},
	'/api/v1/slice/[configId]': {
		api: true,
		cookie: true,
		name: 'slice.slice'
	}
}

function objectFind(obj: object, fn: (value: any, key: any) => boolean): {key: string, value: any} | null {
	let res: {key: string, value: any} | null = null
	objectLoop(obj, (value, key) => {
		const tmp = fn(value, key)
		if (tmp) {
			res = {
				key, value
			}
		}
		return !tmp
	})
	return res
}

// `context` and `next` are automatically typed
export default defineMiddleware(async (context, next) => {
	if (!context.request.url.includes('api')) {
		return next()
	}
	const permission = objectFind(endpointsPermissions, (_, key) => new URLManager(key).toString(context.params as any) === context.url.pathname)
	if (!permission) {
		return buildRFC7807({
			type: 'idk'
		})
	}
	const auth = await validateAuth(context.request, permission.value)
	if (typeof auth === 'object') {
		return auth
	}
	context.locals.authKey = auth

	return next()
})
