import { objectOmit } from '@dzeio/object-util'
import mongoose from 'mongoose'
import type User from '.'
import Client from '../Client'
import Dao from '../Dao'

export default class UserDao extends Dao<User> {

	// @ts-expect-error typing fix
	private model = mongoose.models['User'] as null ?? mongoose.model('User', new mongoose.Schema({
		email: { type: String, required: true },
		password: { type: String, required: true }
	}, {
		timestamps: true
	}))

	public async create(obj: Omit<User, 'id' | 'created' | 'updated'>): Promise<User | null> {
		await Client.get()
		return this.fromSource(await this.model.create(obj))
	}

	public async findAll(query?: Partial<User> | undefined): Promise<User[]> {
		await Client.get()
		if (query?.id) {
			const item = await this.model.findById(new mongoose.Types.ObjectId(query.id))
			if (!item) {
				return []
			}
			return [this.fromSource(item)]
		}
		const resp = await this.model.find(query ? this.toSource(query as User) : {})
		return resp.map(this.fromSource)

	}

	public async update(obj: User): Promise<User | null> {
		await Client.get()

		const query = await this.model.updateOne({
			_id: new mongoose.Types.ObjectId(obj.id)
		}, this.toSource(obj))
		if (query.matchedCount >= 1) {
			obj.updated = new Date()
			return obj
		}
		return null
		// return this.fromSource()
	}

	public async delete(obj: User): Promise<boolean> {
		await Client.get()
		const res = await this.model.deleteOne({
			_id: new mongoose.Types.ObjectId(obj.id)
		})
		return res.deletedCount > 0
	}

	private toSource(obj: User): Omit<User, 'id'> {
		return objectOmit(obj, 'id', 'updated', 'created')
	}

	private fromSource(doc: mongoose.Document<any, any, User>): User {
		return {
			id: doc._id.toString(),
			email: doc.get('email'),
			password: doc.get('password'),
			updated: doc.get('updatedAt'),
			created: doc.get('createdAt')
		}
	}
}
