import { objectOmit } from '@dzeio/object-util'
import type { APIRoute } from 'astro'
import StatusCode from '../../../../../../libs/HTTP/StatusCode'
import { buildRFC7807 } from '../../../../../../libs/RFCs/RFC7807'
import DaoFactory from '../../../../../../models/DaoFactory'

export const post: APIRoute = async ({ params, request, locals }) => {
	const userId = params.userId as string

	const body = request.body

	if (!body) {
		return buildRFC7807({
			title: 'Missing config file'
		})
	}

	const reader = body.getReader()

	const chunks: Array<Uint8Array> = []

	let finished= false
	do {
		const { done, value } = await reader.read()
		finished = done
		if (value) {
			chunks.push(value)
		}
	} while (!finished)

	const buffer = Buffer.concat(chunks)

	const dao = await DaoFactory.get('config').create({
		user: userId,
		type: 'prusa',
		files: [{
			name: 'config.ini',
			data: buffer
		}]
	})
	return locals.responseBuilder
		.status(StatusCode.CREATED)
		.body(objectOmit(dao ?? {}, 'files'))
		.build()
}
